@@ -15,28 +15,39 @@ bestbeta<-function (fit, exclude = NULL, start = NULL,touse=NA){
 #   cat("Just entered bestbeta touse",touse,"\n")
 #   browser()
     start <- rep(0, dim(fit$x)[2])
-#   cat("In bestbeta before raw call to newllk touse",touse,".\n")
+#   message("In bestbeta before raw call to newllk touse",touse)
     llkout <- newllk(start, fit, exclude = exclude,keeponly=touse,justd0=TRUE)
 #   cat("In bestbeta after raw call to newllk\n")
     if(is.na(touse[1])){
        touse <- rep(TRUE, dim(fit$x)[2])
+       names(touse)<-dimnames(fit$x)[[2]]
     }
 #   browser()
     newvar<-array(NA,rep(length(touse),2))
+    dimnames(newvar)<-list(names(touse),names(touse))
+#   print(names(touse))
+#   print.default(newvar)
+    newcoef<-rep(NA,length(touse))
     if(any(touse)){
 #      cat("In bestbeta about to call nlmo touse",touse,"\n")
        aaa<-fit
        aaa$x<-fit$x[,touse,drop=FALSE]
        nlmo<-nlm(newllk,rep(0,sum(touse)),hessian=TRUE,fit=aaa,exclude=exclude,minus=TRUE)
 #      browser()
-       newcoef<-rep(NA,length(touse))
        newcoef[touse]<-nlmo$estimate
        names(newcoef)<-names(touse)
-       temp<-try(solve(nlmo$hessian))
+#      message("In bestbeta Hessian diagonal",paste(diag(nlmo$hessian),collapse=" "))
+#      message("In bestbeta inv Hessian diagonal",paste(diag(solve(nlmo$hessian)),collapse=" "))
+#      if(any(diag(nlmo$hessian)<0)){
+#         message("In bestbeta Hessian should be negative definite")
+#         browser()
+#      }
+       temp<-try(solve(nlmo$hessian),silent=TRUE)
        if(inherits(temp,"try-error")){
           message("Inverting hessian failed")
        }else{
           newvar[touse,touse]<-temp
+#         cat("temp"); print(temp); cat("newvar"); print(newvar)
        }
        score<-NA
 #      print("touse");print(touse)
@@ -46,17 +57,23 @@ bestbeta<-function (fit, exclude = NULL, start = NULL,touse=NA){
        if(inherits(linear.predictors,"try-error")){
           message("Error in linear predictor"); browser()
        }
-       wald.test<-try(newcoef[touse]%*%solve(newvar[touse,touse,drop=FALSE],newcoef[touse]),silent=TRUE)
+       vv<-try(solve(newvar[touse,touse,drop=FALSE],newcoef[touse]),silent=TRUE)
+       if(inherits(vv,"try-error")){
+          message("Inverting matrix for wald test failed")
+          wald.test<-NA
+       }else{
+          wald.test<-newcoef[touse]%*%vv
+#         cat("temp"); print(temp); cat("newvar"); print(newvar)
+       }
     }else{
-       newcoef<-NULL
-       var<-array(0,c(0,0))
-       loglik<-list(d0=0)
+       var<-array(NA,c(length(newcoef),length(newcoef)))
        nlmo<-list(minimum=0,iterations=0)
        score=0
        wald.test=0
        linear.predictors<-rep(0,dim(fit$x)[1])
     }
     out<-list(coefficients=newcoef,var=newvar,loglik=c(llkout$d0,-nlmo$minimum),score=score, wald.test=wald.test, linear.predictors=linear.predictors,iterations=nlmo$iterations)
+#   print(out)
     class(out)<-"coxph"
     return(out)
 }
